require_relative "spec_helper"
require "pry"
require 'pp'

describe Dilute::Modelize do
  let(:dilute_class) do
    Class.new do
      include Dilute::Modelize

      before_save :a_callback
      def a_callback ; true ; end

      define_type(type_name: :abstract) do
        attribute(:data).not_indexed
        attribute(:header).not_indexed
        attribute(:keys).not_indexed# .index_analyzer("keyword")
        attribute(:created_at, :date)
      end
    end
  end

  let(:test_data) { "foo city" }
  let(:new_diluted) { dilute_class.new({data: test_data}) }
  let(:saved) { new_diluted.save ; new_diluted }

  context "validations" do
    let(:dilute_class) do
      Class.new do
        include Dilute::Modelize

        validates_presence_of :data

        define_type(type_name: :abstract) do
          attribute(:data).not_indexed
        end
      end
    end

    it "should validate" do
      expect(dilute_class.new).not_to be_valid
    end

    it "should not save invalid record" do
      destroy_test_index!
      expect(dilute_class.new.save).to be_false
    end
  end

  it "should have accessors for new object" do
    expect(new_diluted.data).to eq(test_data)
  end

  it "should save data" do
    expect(new_diluted.id).to be_nil
    new_diluted.save
    expect(new_diluted.id).not_to be_nil
  end

  it "should retrieve saved data" do
    expect(dilute_class.find(saved.id).data).to eq(test_data)
  end

  it "I have no idea what I'm doing." do
    true
  end

  context "searching" do
    before { destroy_test_index! }
    it "should find docs" do
      saved
      dilute_class.refresh
      all = dilute_class.all
      expect(all.to_a).to have(1).item
      expect(all.first.data).to eq(test_data)
    end

    it "should find docs matching pattern" do
      dilute_class.new({keys: %w(taga tagb tagc)}).save
      dilute_class.new({keys: %w(tagb tagc)}).save
      dilute_class.refresh
      search = dilute_class.all.match(keys: "taga")
      expect(search.to_a).to have(1).item
    end

    it "should find docs matching very short word" do
      dilute_class.new({keys: %w(a)}).save
      dilute_class.new({keys: %w(b)}).save
      dilute_class.refresh
      search = dilute_class.all.match(keys: "a")
      expect(search.to_a).to have(1).item
    end

  end

  context "infering the index name" do
    let(:dilute_class) do
      Class.new do
        include Dilute::Modelize
        define_singleton_method :name do
          "TestNameForDilute"
        end

        define_type do
          attribute(:data).not_indexed
        end
      end
    end

    it "should save record" do
      new_diluted.save
      expect(new_diluted.attributes["_index"]).to eq "default_dilute_index_name"
      expect(new_diluted.attributes["_type"]).to eq "test_name_for_dilutes"
    end
  end

  context "callbacks" do
    it "should not save when callback is false" do
      destroy_test_index!
      def new_diluted.a_callback ; false ; end
      expect(new_diluted.save).to be_false
      dilute_class.refresh
      expect(dilute_class.all.first).to be_nil
    end
  end
end
